<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

class AriSmartContentFormattersHelper
{
	static function getFormatter($formatterName)
	{
		$formatterPath = 'Application.ARISmartContent.Formatters.' . ucfirst($formatterName) . 'Formatter';

		AriKernel::import($formatterPath);

		$formatterClass = 'AriSmartContentFormatter' . ucfirst($formatterName);
		if (!class_exists($formatterClass))
		{
			JError::raiseError(
				500,
				sprintf(
					'ARI Smart Content: "%1$s" formatter does not exist',
					$formatterClass
				)
			);

			return null;
		}

		$formatter = new $formatterClass();

		return $formatter;
	}
	
	static function getColumnSettings($columnsSettings, $key, $idx)
	{
		$colSettings = null;
		if (empty($columnsSettings))
			return $colSettings;
		
		if (array_key_exists($key, $columnsSettings))
			$colSettings = $columnsSettings[$key];
		else if (array_key_exists($idx, $columnsSettings))
			$colSettings = $columnsSettings[$idx];
			
		return $colSettings;
	}
	
	static function getHiddenColumns($columns, $columnsSettings)
	{
		$hiddenColumns = array();
		
		if (empty($columns) || empty($columnsSettings))
			return $hiddenColumns;
			
		$columnsNames = array();
		foreach ($columns as $column)
		{
			$columnsNames[] = is_array($column) ? $column['name'] : $column;
		}

		foreach ($columnsSettings as $key => $colSettings)
		{
			if (empty($colSettings['hidden']))
				continue ;

			if (is_int($key))
				$hiddenColumns[] = $key;
			else 
			{
				$idx = array_search($key, $columnsNames);
				if ($idx !== false)
					$hiddenColumns[] = $idx;
			}
		}
		
		$hiddenColumns = array_unique($hiddenColumns);
		sort($hiddenColumns);
		
		return $hiddenColumns;
	}
	
	function recalculateIndex($index, $hiddenColumns)
	{
		if (!is_array($hiddenColumns) || count($hiddenColumns) == 0)
			return $index;

		$diff = 0;
		foreach ($hiddenColumns as $hiddenColumn)
		{
			if ($hiddenColumn >= $index)
				break ;

			++$diff;
		}
		
		return $index - $diff;
	}
}