<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

AriKernel::import('Application.ARISmartContent.Formatters.FormatterBase');
AriKernel::import('SimpleTemplate.SimpleTemplate');
AriKernel::import('Web.JSON.JSON');
AriKernel::import('Parameters.ParametersHelper');

class AriSmartContentFormatterTable extends AriSmartContentFormatterBase
{
	function formatData($data, $formatter, $columns, $columnsSettings)
	{
		if (empty($data))
			return $data;
		
		$flatParams = AriParametersHelper::flatParametersToArray($formatter);
		$revertData = AriUtils::getParam($flatParams, 'RevertData');
		$revertDataEnable = (bool)AriUtils::getParam($revertData, 'enable');
		if ($revertDataEnable)
			$data = $this->revertData($data);
			
		$graphTransform = AriUtils::getParam($flatParams, 'GraphTransform');
		$graphTransformEnable = (bool)AriUtils::getParam($graphTransform, 'enable');
		if ($graphTransformEnable)
			$data = $this->graphTransform($data, $graphTransform);

		foreach ($data as $dataItemKey => &$dataItem)
		{
			$colIdx = -1;
			$originalDataItem = $dataItem;
			foreach ($dataItem as $colKey => $value)
			{
				++$colIdx;
				$columnSettings = AriSmartContentFormattersHelper::getColumnSettings($columnsSettings, $colKey, $colIdx);

				if (empty($columnSettings))
					continue;

				if (!empty($columnSettings['format']))
				{
					$dataItem[$colKey] = AriSimpleTemplate::parse(
						$columnSettings['format'], 
						array(
							'value' => $value,
							'columns' => $originalDataItem,
							'REQUEST' => $_REQUEST
						),
						true
					);
				}				
			}			
		}
		
		return $data;
	}

	function revertData($data)
	{
		if (empty($data))
			return $data;
	
		$revertedData = array();
		$colKeys = array_keys($data[0]);
		$keyField = array_shift($colKeys);
		$colCount = count($colKeys);

		for ($i = 0; $i < $colCount; $i++)
		{
			$revertedData[] = array('&nbsp;' => $colKeys[$i]);
		}
		
		$rowIdx = 0;
		foreach ($data as $dataItem)
		{
			$colIdx = 0;
			foreach ($colKeys as $key)
			{
				$revertedData[$colIdx][$dataItem[$keyField]] = $dataItem[$key];

				++$colIdx;
			}
			++$rowIdx;
		}

		return $revertedData;
	}
	
	function graphTransform($data, $graphTransform)
	{
		if (empty($data))
			return $data;
			
		$xAxis = trim(AriUtils::getParam($graphTransform, 'xAxis', ''));
		$yAxis = trim(AriUtils::getParam($graphTransform, 'yAxis', ''));
		$valueCol = trim(AriUtils::getParam($graphTransform, 'value', ''));
		$defValue = AriUtils::getParam($graphTransform, 'defValue');

		if (empty($xAxis) || empty($yAxis) || empty($valueCol) || $xAxis == $yAxis)
			return $data;
		
		$prepareData = array();
		$yValues = array();
		foreach ($data as $dataItem)
		{
			$xVal = $dataItem[$xAxis];
			$yVal = $dataItem[$yAxis];
			$val = $dataItem[$valueCol];

			if (!isset($prepareData[$xVal]))
				$prepareData[$xVal] = array();

			$yValues[$yVal] = $yVal;
			$prepareData[$xVal][$yVal] = $val;
		}

		$newData = array();
		foreach ($yValues as $yVal)
		{
			$row = array();
			$row[] = $yVal;
			foreach ($prepareData as $key => $val)
			{
				$row[$key] = isset($val[$yVal]) ? $val[$yVal] : $defValue;
			}

			$newData[] = $row;
		}

		return $newData;
	}
	
	function getColumnsSettings($formatter, $columns)
	{
		$columnsSettings = array();

		$settings = AriUtils::getParam($formatter, 'ColumnsSettings');
		$columnsSortable = (bool)AriUtils::getParam($formatter, 'ColumnsSortable');
		$columnsSearchable = (bool)AriUtils::getParam($formatter, 'ColumnsSearchable');
		$usedIdx = array();

		$columnsNames = array();
		foreach ($columns as $column)
		{
			$columnsNames[] = $column['name'];
		}
		
		if ($settings)
		{
			$settings = json_decode($settings, true);
			if (!empty($settings))
			{
				foreach ($settings as $colParam)
				{
					$id = AriUtils::getParam($colParam, 'id');
					if (is_null($id) || strlen($id) == 0)
						continue ;
	
					$idx = -1;
					if ($id == '' . intval($id, 10))
					{
						$idx = $colParam['id'] = intval($id, 10);
					}
					else
					{
						$idx = array_search($id, $columnsNames);
						if ($idx === false)
							$idx = -1;
					}
					
					if ($idx !== -1)
						$usedIdx[] = $idx;

					if (!isset($colParam['sortable']) || $colParam['sortable'] == '-1')
						$colParam['sortable'] = $columnsSortable;

					if (!isset($colParam['searchable']) || $colParam['searchable'] == '-1')
						$colParam['searchable'] = $columnsSearchable;
	
					$colParam['idx'] = $idx;
	
					if (empty($colParam['class']))
						$colParam['class'] = '';
	
					if (!empty($colParam['format']))
						$colParam['format'] = trim($colParam['format']);
	
					$colClass = $this->getColClass($colParam['id']);
						
					$colParam['class'] .= ' ' . $colClass;
					$colParam['systemClass'] = $colClass;
					$columnsSettings[$colParam['id']] = $colParam;
				}
			}	
		}

		if (is_array($columns))
		{
			foreach ($columnsNames as $colIdx => $column)
			{
				if (in_array($colIdx, $usedIdx))
					continue ;
					
				if (!isset($columnsSettings[$colIdx]))
					$columnsSettings[$colIdx] = array('idx' => $colIdx);
					
				$columnsSettings[$colIdx]['sortable'] = $columnsSortable;
				$columnsSettings[$colIdx]['searchable'] = $columnsSearchable;
			}
		}

		return $columnsSettings;
	}
	
	function isDataTransformed($formatter)
	{
		$flatParams = AriParametersHelper::flatParametersToArray($formatter);

		$revertData = AriUtils::getParam($flatParams, 'RevertData');
		$revertDataEnable = (bool)AriUtils::getParam($revertData, 'enable');

		$graphTransform = AriUtils::getParam($flatParams, 'GraphTransform');
		$graphTransformEnable = (bool)AriUtils::getParam($graphTransform, 'enable');
		
		return ($revertDataEnable || $graphTransformEnable);
	}

	protected function getColClass($id)
	{
		return 'col_' . substr(md5($id), 0, 8);
	}	
}