<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

AriKernel::import('Plugin.ContentPlugin');

class AriSmartContentNodeListPlugin extends AriContentPlugin
{
	var $_tag;
	var $_nodeList = array();
	var $_singleRoot = true;

	function getIsSingleRoot()
	{
		return $this->_singleRoot;
	}
	
	function getNodeList($content)
	{
		$this->_nodeList = array();
		$content = $this->prepareContent($content);
		$this->parse($content);
		$this->prepareNodeList();

		return $this->_nodeList;
	}

	function prepareContent($content)
	{
		if (!$this->isValidCode($content))
			return $content;

		$tag = $this->getTag();
		$content = str_replace('{' . $tag . '}', '{' . $tag . ' }', $content);
		$openTag = '{' . $tag . ' ';
		$closeTag = '{/' . $tag . '}';

		$closeTagPos = strpos($content, $closeTag);
		if ($closeTagPos === false)
			return $content;

		if (strpos($content, $openTag) === false)
			return $content;

		$openTagPos = -1;
		$idList = array();
		$correction = 0;
		while ($openTagPos !== false) 
		{
			$openTagPos = strpos($content, $openTag, $openTagPos + 1);

			if ($openTagPos !== false && $openTagPos < $closeTagPos) 
			{
				$currentId = uniqid('', false);
				$parentId = count($idList) > 0 ? $idList[count($idList) - 1] : null;
				$idList[] = $currentId;

				$content = $this->modifyContent($content, $openTagPos, $currentId, $parentId, $correction);

				$closeTagPos += $correction;
			} 
			else if ($closeTagPos !== false) 
			{
				while ($closeTagPos !== false && ($openTagPos === false || $closeTagPos < $openTagPos)) 
				{
					$closeTagPos = strpos($content, $closeTag, $closeTagPos + 1);

					array_pop($idList);
				}

				if ($openTagPos !== false)
				{
					$currentId = uniqid('', false);
					$parentId = count($idList) > 0 ? $idList[count($idList) - 1] : null;
					$idList[] = $currentId;

					$content = $this->modifyContent($content, $openTagPos, $currentId, $parentId, $correction);

					$closeTagPos += $correction;
				}
			}
		}			

		return $content;
	}
	
	function prepareNodeList()
	{
		$rootNodes = array();
		
		foreach ($this->_nodeList as $nodeId => &$node)
		{
			$parentId = $node['parentId'];
			if ($parentId)
			{
				$parentNode =& $this->_nodeList[$parentId];
				$parentNode['children'][] = $node;
			}
			else
				$rootNodes[] = $node;
		}
		
		if ($this->getIsSingleRoot() && count($rootNodes) > 1)
			$rootNodes = array($rootNodes[0]);

		$this->_nodeList = $rootNodes;
	}

	function modifyContent($content, $pos, $id, $parentId, &$correction)
	{
		$tag = '{' . $this->getTag() . ' ';
		
		$idAttrs = '_node_id="' . $id . '"';
		if ($parentId)
			$idAttrs .= ' _node_parent_id="' . $parentId . '"';
			
		$idAttrs .= ' ';
		
		$content = substr($content, 0, $pos) . $tag . $idAttrs . substr($content, $pos + strlen($tag));
		$correction = strlen($idAttrs);

		return $content;
	}

	function contentHandler($params, $content, $sourceContent) 
	{
		$nodeId = AriUtils::getParam($params, '_node_id');
		$parentId = AriUtils::getParam($params, '_node_parent_id', 0);

		unset($params['_node_id']);
		unset($params['_node_parent_id']);

		$this->_nodeList[$nodeId] = array(
			'nodeId' => $nodeId,
			'parentId' => $parentId,
			'params' => $params,
			'content' => $content,
			'children' => array()
		);

		return '';
	}
}