<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

AriKernel::import('Application.ARISmartContent.Providers.ProviderBase');
AriKernel::import('DB.ADODBLite');
AriKernel::import('SimpleTemplate.SimpleTemplate');
AriKernel::import('Utils.ArrayHelper');
AriKernel::import('DB.SQLParser.SQLParser');
AriKernel::import('DB.SQLParser.SQLCreator');
AriKernel::import('Data.DataFilter');

class AriSmartContentProviderDatabaseQuery extends AriSmartContentProviderBase
{
	function getData($source, $filter)
	{
		$query = $this->prepareQuery($source->Params->query, $filter, false, $source);
		$data = $this->executeQuery($query, $source);

		if (is_array($data) && count($data) > 0)
		{
			$connectionParams = $source->ProviderSettings->Params;
			if ($connectionParams->connection == 'external')
			{
				$dbType = AriUtils::getParam($connectionParams, 'external_dbtype', 'mysql');
				$charsetName = AriUtils::getParam($connectionParams, 'external_charset', '');
				
				if ($charsetName && strpos($dbType, 'mysql') !== 0)
				{
					foreach ($data as &$dataItem)
					{
						foreach ($dataItem as &$val)
						{
							$val = @iconv($charsetName, 'UTF-8', $val);
						}
					}
				}
			}
		}
		
		return $data;
	}
	
	function executeQuery($query, $source)
	{
		$data = null;

		$connectionParams = $source->ProviderSettings->Params;

		if ($connectionParams->connection == 'external')
			$data = $this->getDataFromExternalSource($source, $query);
		else
			$data = $this->getDataFromDefaultSource($source, $query);
			
		return $data;
	}

	function getTotalRecords($source, $filter)
	{
		$query = $this->prepareQuery($source->Params->query, $filter, true, $source);
		$parser = new PHPSQLParser($query);
		$parsedResults = $parser->parsed;
		$parsedResults['SELECT'] = array(
			array(
				'expr_type' => 'aggregate_function',
				'alias' => array(
					'as' => '1',
					'name' => 'TotalRecords',
					'base_expr' => 'AS TotalRecords'
				),
	
				'base_expr' => 'COUNT',
				'sub_tree' => array(
					array(
						'expr_type' => 'colref',
						'base_expr' => '*',
						'sub_tree' => ''
					)
				)
			)
		);
		unset($parsedResults['LIMIT']);

		$sqlCreator = new PHPSQLCreator();
		$query = $sqlCreator->create($parsedResults);

		$data = $this->executeQuery($query, $source);
		$totalRecords = 0;
		if (isset($data[0]['TotalRecords']))
			$totalRecords = $data[0]['TotalRecords'];

		return $totalRecords;
	}

	function getColumns($source)
	{
		$columns = array();
		if ($source->Formatter && $source->Formatter->Type)
		{
			$formatter = AriSmartContentFormattersHelper::getFormatter($source->Formatter->Type);

			if ($formatter->isDataTransformed($source->Formatter->Data))
			{
				$data = $this->getData($source, null);
				$data = $formatter->formatData($data, $source->Formatter->Data, null, null);

				if (count($data) == 0)
					return $columns;

				$columnNames = $data[0];
				$columnNames = array_keys($columnNames);
				
				foreach ($columnNames as $columnName)
				{
					$columns[] = array(
						'name' => $columnName
					);
				}

				return $columns;
			}
		}

		$query = $this->prepareQuery($source->Params->query, null, false, $source);
		$parser = new PHPSQLParser($query);
		$parsedResults = $parser->parsed;

		if (is_array($parsedResults['SELECT']))
		{
			$getColumnsFromData = false;
			foreach ($parsedResults['SELECT'] as $dbCol)
			{
				if (strpos($dbCol['base_expr'], '*') !== false)
					$getColumnsFromData = true;
			}
			
			if ($getColumnsFromData)
			{
				$colFilter = new AriDataFilter();
				$colFilter->setConfigValue('startOffset', 0);
				$colFilter->setConfigValue('limit', 1);
				
				$data = $this->getData($source, $colFilter);

				if (count($data) == 0)
					return $columns;

				$columnNames = $data[0];
				$columnNames = array_keys($columnNames);

				foreach ($columnNames as $columnName)
				{
					$columns[] = array(
						'name' => $columnName
					);
				}
			}
			else
			{
				foreach ($parsedResults['SELECT'] as $dbCol)
				{
					$col = $colOriginal = $dbCol['base_expr'];
					if (!empty($dbCol['alias']['name']))
						$col = $dbCol['alias']['name'];
					else
					{
						$col = explode('.', $col);
						$col = $col[count($col) - 1];
					}
	
					if ($dbCol['expr_type'] != 'colref')
					{
						unset($dbCol['alias']);
						$sqlCol = array(
							'SELECT' => array($dbCol)
						);
						$sqlCreator = new PHPSQLCreator();
						try
						{
							$colQuery = $sqlCreator->create($sqlCol);
							$colOriginal = str_replace('SELECT ', '', $colQuery);
						}
						catch (Exception $ex)
						{
							$colOriginal = null;
						}
					}
	
					$columns[] = array(
						'original' => $colOriginal,
						'name' => trim($col, '`')
					);
				}
			}
		}

		return $columns;
	}
	
	protected function getDataFromExternalSource($source, $query)
	{
		$connectionParams = $source->ProviderSettings->Params;
		
		$debug = (bool)AriUtils::getParam($connectionParams, 'debug', false);
		$dbHost = AriUtils::getParam($connectionParams, 'external_dbhost', 'localhost');
		$dbName = AriUtils::getParam($connectionParams, 'external_dbname', '');
		$dbUser = AriUtils::getParam($connectionParams, 'external_dbuser', 'root');
		$dbPass = AriUtils::getParam($connectionParams, 'external_dbpass', '');
		$dbType = AriUtils::getParam($connectionParams, 'external_dbtype', 'mysql');
		$persistent = (bool)AriUtils::getParam($connectionParams, 'external_persistent', false);
		$charsetName = AriUtils::getParam($connectionParams, 'external_charset', '');

		$conn = false;
		if (AriADODBLite::isAllowedDbType($dbType) && !empty($dbHost) && !empty($dbName))
		{
			$conn = $persistent
				? AriADODBLite::createPConnection($dbHost, $dbUser, $dbPass, $dbName, null, $dbType)
				: AriADODBLite::createNConnect($dbHost, $dbUser, $dbPass, $dbName, null, $dbType);
			if (!$conn || !$conn->IsConnected()) 
				$conn = false;
			else
			{
				if ($debug)
					$conn->debug = $debug;
				if ($charsetName)
				{
					if (strpos($dbType, 'mysql') === 0)
					{
						$db = JFactory::getDBO();
						$conn->Execute('SET NAMES ' . $db->Quote($charsetName));
					}
				}
			}
		}

		if (!$conn)
		{
			if ($debug)
				JError::raiseWarning(0, JText::_("ARI Data Tables: Couldn't connect to external database."));

			return null;
		}

		global $ADODB_FETCH_MODE;

		$ADODB_FETCH_MODE = ADODB_FETCH_ASSOC;
		$data = $conn->GetArray($query);
		if ($conn->ErrorNo())
		{
			$data = null;
		}

		if ($conn && $conn->IsConnected()) 
			$conn->Close();

		return $data;
	}

	protected function getDataFromDefaultSource($source, $query)
	{
		$debug = (bool)AriUtils::getParam($source->ProviderSettings->Params, 'debug', false);
		$db = JFactory::getDBO();
		if ($debug)
			$db->debug($debug);

		$db->setQuery($query);
		$data = $db->loadAssocList();
		if ($db->getErrorNum())
		{
			$data = null;
		}
		
		return $data;
	}
	
	protected function prepareQuery($query, $filter = null, $onlyWhere = false, $source = null)
	{
		$query = $this->replacePredefinedVariables($query);
		$query = $this->applyFilterToQuery($query, $filter, $onlyWhere, $source);

		return $query;
	}
	
	protected function replacePredefinedVariables($query)
	{
		$query = str_replace(array('&gt;', '&lt;', '&#039;'), array('>', '<', '\''), $query);
		if (strpos($query, '{$') === false)
			return $query;

		$db = JFactory::getDBO();
		$user = JFactory::getUser();

		$params = array(
			'UserId' => $user->get('id'),
			'UserGroupId' => $user->get('gid'),
			'UserType' => $db->Quote($user->get('usertype')),
			'UserName' => $db->Quote($user->get('name')),
			'UserLogin' => $db->Quote($user->get('username')),
			'UserBlock' => $user->get('block'),
			'UserEmail' => $db->Quote($user->get('email'))
		);

		if (strpos($query, '{$REQUEST:') !== false)
		{
			$request = $_REQUEST;

			AriArrayHelper::walkRecursive($request, array(&$db, 'Quote'));
			$params['REQUEST'] = $request;
		}

		if (strpos($query, '{$VAR:') !== false && isset($GLOBALS['ASC_VARS']))
		{
			$vars = $GLOBALS['ASC_VARS'];
	
			AriArrayHelper::walkRecursive($vars, array(&$db, 'Quote'));
			$params['VAR'] = $vars;
		}

		$query = AriSimpleTemplate::parse($query, $params, true);

		return $query;
	}
	
	protected function applyFilterToQuery($query, $filter, $onlyWhere = false, $source = null)
	{
		if (empty($filter))
			return $query;

		$parser = new PHPSQLParser($query);
		$parsedResults = $parser->parsed;

		unset($parsedResults['LIMIT']);

		$db = JFactory::getDBO();

		$filterPatterns = $filter->getConfigValue('filter');
		if (is_array($filterPatterns) && count($filterPatterns) > 0)
		{
			$searchExpr = '';
			$subTreeSearch = array();

			foreach ($filterPatterns as $filterPattern)
			{
				if (count($subTreeSearch) > 0)
					$subTreeSearch[] = array(
						'expr_type' => 'operator',
						'base_expr' => 'OR',
						'sub_tree' => null
					);

				$subTreeSearch[] = array(
					'expr_type' => 'colref',
					'base_expr' => $this->quoteName($filterPattern['field'], $source),
					'sub_tree' => null
				);

				$subTreeSearch[] = array(
					'expr_type' => 'operator',
					'base_expr' => $filterPattern['op'],
					'sub_tree' => null
				);
				
				$subTreeSearch[] = array(
					'expr_type' => 'const',
					'base_expr' => $db->Quote('%' . $filterPattern['value'] . '%'),
					'sub_tree' => null
				);
			}

			if (!isset($parsedResults['WHERE']))
				$parsedResults['WHERE'] = array();
	
			if (count($parsedResults['WHERE']) > 0)
			{
				$parsedResults['WHERE'][] = array(
                    'expr_type' => 'operator',
                   	'base_expr' => 'AND',
                   	'sub_tree' => null
				);
			}
				
			$parsedResults['WHERE'][] = array(
				'expr_type' => 'bracket_expression',
				'base_expr' => $searchExpr,
				'sub_tree' => $subTreeSearch
			);
		}
		
		if (!$onlyWhere)
		{
			$startOffset = $filter->getConfigValue('startOffset');
			$limit = $filter->getConfigValue('limit');

			if (!is_null($startOffset) && $startOffset > 0)
			{
				if (!isset($parsedResults['LIMIT']))
					$parsedResults['LIMIT'] = array();
				
				$parsedResults['LIMIT']['offset'] = $startOffset;
			}

			if (!is_null($limit) && $limit > 0)
			{
				if (!isset($parsedResults['LIMIT']))
					$parsedResults['LIMIT'] = array('offset' => 0);

				$parsedResults['LIMIT']['rowcount'] = $limit;
			}

			$sortField = $filter->getConfigValue('sortField');
			if (!empty($sortField))
			{
				$sortDir = $filter->getConfigValue('sortDirection');

				$parsedResults['ORDER'] = array();
				$parsedResults['ORDER'][] = array(
					'expr_type' => 'colref',
					'base_expr' => $this->quoteName($sortField, $source),
					'sub_tree' => null, 
					'direction' => $sortDir
				);

				$secondarySorting = $filter->getConfigValue('secondarySorting');
				if (is_array($secondarySorting))
				{
					foreach ($secondarySorting as $sortItem)
					{
						$parsedResults['ORDER'][] = array(
							'expr_type' => 'colref',
							'base_expr' => $this->quoteName($sortItem['field'], $source),
							'sub_tree' => null, 
							'direction' => $sortItem['dir']
						);
					}
				}
			}
		}

		$sqlCreator = new PHPSQLCreator();
		$query = $sqlCreator->create($parsedResults);

		return $query;	
	}

    protected function quoteName($name, $source = null)
    {
        $qName = $name;
        $connectionParams = !is_null($source) ? $source->ProviderSettings->Params : null;

        if (
                is_null($connectionParams) ||
                $connectionParams->connection != 'external'
        )
        {
            $db = JFactory::getDBO();

            $quote_chars = $db->quoteName('');
            if ($quote_chars)
                $qName = trim($qName, $quote_chars);

            $qName = $db->quoteName($qName);
        }
        else
        {
            $dbType = AriUtils::getParam($connectionParams, 'external_dbtype', 'mysql');
            $dbDriver = ADONewConnection($dbType);

            if ($dbDriver->nameQuote)
            {
                $qName = trim($qName, $dbDriver->nameQuote);

                $qName = $dbDriver->nameQuote . $qName . $dbDriver->nameQuote;
            }
        }

        return $qName;
    }

	function isSupportServerSide($source)
	{
		$isSupported = true;
		if ($source->Formatter && $source->Formatter->Type)
		{
			$formatter = AriSmartContentFormattersHelper::getFormatter($source->Formatter->Type);

			if ($formatter->isDataTransformed($source->Formatter->Data))
				$isSupported = false;
		}

		return $isSupported;
	}
}