<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

AriKernel::import('Application.ARISmartContent.Providers.ProviderBase');
AriKernel::import('Application.ARISmartContent.Utils.ApplicationHelper');
AriKernel::import('Flickr.Flickr');
AriKernel::import('Utils.Utils');
AriKernel::import('SimpleTemplate.SimpleTemplate');
AriKernel::import('Web.JSON.JSON');
AriKernel::import('Parameters.ParametersHelper');

class AriSmartContentProviderFlickrImages extends AriSmartContentProviderBase
{
	var $_typeMapping = array(
		'thumbnail' => 't',
		'square' => 'sq',
		'small' => 's',
		'medium' => 'm',
		'large' => 'l'
	);

	function getData($source, $filter)
	{
		$data = array();
		$params = $source->Params;
		$flickrParams = $source->ProviderSettings->Params;
		$cachePeriod = intval(AriUtils::getParam($params, 'cachePeriod', 10), 10);
		$flickr = $this->getFlickr($flickrParams, $cachePeriod);
		$src = AriUtils::getParam($params, 'source_type');
		$paramsTree = AriParametersHelper::flatParametersToArray($params);
		$srcParams = isset($paramsTree['source'][$src]) ? $paramsTree['source'][$src] : array();

		switch ($src)
		{
			case 'collection':
				$collectionIds = explode(';', $srcParams['collectionId']);
				$data = $this->getCollectionsData($collectionIds, $srcParams['userId'], $params, $flickr);
				break;

			case 'group':
				$groupIds = explode(';', $srcParams['groupId']);
				$data = $this->getGroupData($groupIds, $params, $flickr);
				break;
			
			case 'photoset':
				$photosetIds = explode(';', $srcParams['photosetId']);
				$data = $this->getPhotosetsData($photosetIds, $params, null, $flickr);
				break;

			case 'user':
				$userIds = explode(';', $srcParams['userId']);
				$data = $this->getUsersData($userIds, $params, $flickr);
				break;

			case 'recentphotos':
				$userIds = explode(';', $srcParams['userId']);
				$data = $this->getRecentPhotoData($userIds, $params, $flickr);
				break;
		}
		
		if (isset($data['photos']))
			$data = $data['photos'];

		return $data;
	}

	private function getFlickr($params, $cachePeriod = 0)
	{
		$apiKey = $params->apikey;
		$secretKey = !empty($params->secret) ? $params->secret : null;
		$flickr = new phpFlickr($apiKey, $secretKey);

		if (!empty($params->token))
			$flickr->setToken($params->token);

		$helper = AriSmartContentApplicationHelper::getInstance();
		$cacheFolder = $helper->getCacheFolder();

		if ($cachePeriod > 0)
			$flickr->enableCache('fs', $cacheFolder, $cachePeriod);

		return $flickr;
	}

	private function getGroupData($groupIds, $params, $flickr)
	{
		$data = array('photos' => array());
		$photos = array();
		
		$count = $this->getCount($params);
		foreach ($groupIds as $groupId)
		{
			if (empty($groupId))
				continue ;
				
			$groupPhotos = $flickr->groups_pools_getPhotos($groupId, null, null, 'date_upload,last_update, url_sq, url_t, url_s, url_m, url_o, url_l', $count);
			$photosetData = $this->getPhotosetData($groupPhotos, $params);
			if (!empty($photosetData))
			{
				foreach ($photosetData as $key => $value)
				{
					$photos[$key] = $value;
				}
			}
		}
		
		$data['photos'] = $photos;
		
		return $data;
	}
	
	private function getCollectionsData($collectionIds, $userId, $params, $flickr)
	{
		$data = array();
		$photosets = array();
		$photosetsData = array();
		
		foreach ($collectionIds as $collectionId)
		{
			if (empty($collectionId))
				continue ;

			$collection = $flickr->collections_getTree($collectionId, $userId);
			if (empty($collection['collections']['collection'][0]['set']))
				continue ;

			$sets = $collection['collections']['collection'][0]['set'];
			foreach ($sets as $photoset)
			{
				$photosetId = $photoset['id'];
				$photosets[] = $photoset['id'];
				$photosetsData[$photosetId] = $photoset;
			}
		}

		if (count($photosets) > 0)
			$data = $this->getPhotosetsData($photosets, $params, $photosetsData, $flickr);
		
		return $data;
	}
	
	private function getUsersData($userIds, $params, $flickr)
	{
		$data = array();
		$photosets = array();
		$photosetsData = array();

		foreach ($userIds as $userId)
		{
			if (empty($userId))
				continue ;
				
			$flickrPhotosets = $flickr->photosets_getList($userId);
			if (!empty($flickrPhotosets['photoset']))
			{
				foreach ($flickrPhotosets['photoset'] as $photoset)
				{
					$photosetId = $photoset['id'];
					$photosets[] = $photoset['id'];
					$photosetsData[$photosetId] = $photoset;
				}
			}
		}
		
		if (count($photosets) > 0)
			$data = $this->getPhotosetsData($photosets, $params, $photosetsData, $flickr);

		return $data;
	}
	
	private function getRecentPhotoData($userIds, $params, $flickr)
	{
		$data = array();
		$photos = array();

		$count = $this->getCount($params);
		foreach ($userIds as $userId)
		{
			if (empty($userId))
				continue ;

			$flickrPhotos = $this->searchPhotos(
				array(
					'user_id' => $userId, 
					'per_page' => $count, 
					'media' => 'photos', 
					'extras' => 'date_upload,last_update, url_sq, url_t, url_s, url_m, url_o, url_l'
				), 
				$params, 
				$flickr
			);
			$flickrPhotos = $this->getRandomItems($flickrPhotos, $params);
			if (count($flickrPhotos) > 0) $photos = array_merge($photos, $flickrPhotos);
		}

		$data['photos'] = $photos;

		return $data;
	}
	
	private function searchPhotos($args, $params, $flickr)
	{
		$photos = array();
		
		$flickrData = $flickr->photos_search($args);
		$flickrData = $this->getPhotosetData($flickrData, $params);
		if (!empty($flickrData))
		{
			foreach ($flickrData as $key => $value)
			{
				$photos[$key] = $value;
			}
		}
		
		return $photos;
	}
	
	private function getPhotosetsData($photosetIds, $params, $photosets, $flickr)
	{
		$data = array('photos' => array(), 'photosets' => array());
		$photos = array();

		$count = $this->getCount($params);
		foreach ($photosetIds as $photosetId)
		{
			if (empty($photosetId))
				continue ;

			$flickrPhotoset = $flickr->photosets_getPhotos($photosetId, 'date_upload, last_update, url_sq, url_t, url_s, url_m, url_l,url_o, url_l', null, $count);
			$photosetData = $this->getPhotosetData($flickrPhotoset, $params);
			$photosetData = $this->getRandomItems($photosetData, $params);
			if (!empty($photosetData))
			{
				foreach ($photosetData as $key => $value)
				{
					$photos[$key] = $value;
				}
			}

			$data['photosets'][$photosetId] = isset($photosets[$photosetId])
				? $photosets[$photosetId]
				: $flickr->photosets_getInfo($photosetId);
		}
		
		$data['photos'] = $photos;

		return $data;
	}
	
	private function getPhotosetData($photoset, $params)
	{
		$data = array();
		if (empty($photoset))
			return $data;
		
		$photos = null;
		$photoset = AriUtils::getParam($photoset, 'photoset', $photoset);
		if ($photoset) $photos = AriUtils::getParam($photoset, 'photo');
		if (empty($photos))
			return $data;

		$photosetId = AriUtils::getParam($photoset, 'id', null);
		$previewSize = AriUtils::getParam($params, 'previewSize', 'thumbnail');
		$navSize = AriUtils::getParam($params, 'navSize', 'thumbnail');
		$itemSize = $params->itemSize;

		foreach ($photos as $photo)
		{
			$previewUrl = $previewSize ? @phpFlickr::buildPhotoURL($photo, $previewSize) : null;
			$navUrl = $navSize ? @phpFlickr::buildPhotoURL($photo, $navSize) : null;
			$title = str_replace('"', '&quot;', AriUtils::getParam($photo, 'title', ''));

			$data[$photo['id']] = array(
				'photosetId' => $photosetId,
				'dateUpload' => AriUtils::getParam($photo, 'dateupload'),
				'lastUpdate' => AriUtils::getParam($photo, 'lastupdate'),
				'Item_Url' => @phpFlickr::buildPhotoURL($photo, $itemSize),
				'Item_Width' => $this->getPhotoWidth($photo, $itemSize),
				'Item_Height' => $this->getPhotoHeight($photo, $itemSize),
				'Preview_Url' => $previewUrl,
				'Preview_Width' => $previewSize ? $this->getPhotoWidth($photo, $previewSize) : 0,
				'Preview_Height' => $previewSize ? $this->getPhotoHeight($photo, $previewSize) : 0,
				'Nav_Url' => $navUrl,
				'Nav_Width' => $navSize ? $this->getPhotoWidth($photo, $navSize) : 0,
				'Nav_Height' => $navSize ? $this->getPhotoHeight($photo, $navSize) : 0,
				'Title' => $title
			);
		}

		return $data;
	}
	
	private function getShortImageType($type)
	{		
		return isset($this->_typeMapping[$type]) ? $this->_typeMapping[$type] : $type;
	}
	
	private function getPhotoWidth($data, $type)
	{
		$type = $this->getShortImageType($type);

		return intval(AriUtils::getParam($data, 'width_' . $type), 10);
	}
	
	private function getPhotoHeight($data, $type)
	{
		$type = $this->getShortImageType($type);
		
		return intval(AriUtils::getParam($data, 'height_' . $type), 10);
	}
	
	/* temp *
	 */
	function getCount($params, $checkRandom = true)
	{
		if ($checkRandom)
		{
			$random = (bool)AriUtils::getParam($params, 'random');
			if ($random)
				return null;
		}

		$count = intval(AriUtils::getParam($params,'count'), 10);
		if ($count < 1) $count = null;
		
		return $count;
	}

	function getRandomItems($data, $params)
	{
		$random = (bool)AriUtils::getParam($params, 'random');
		if (!$random)
			return $data;
			
		$count = intval(AriUtils::getParam($params,'count'), 10);
		$keys = array_keys($data);
		shuffle($keys);
		if ($count > 0)
			$keys = array_slice($keys, 0, $count);
			
		$randData = array();
		foreach ($keys as $key)
		{
			$randData[$key] = $data[$key];
		}

		return $randData;
	}
}