<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

AriKernel::import('Application.ARISmartContent.Providers.ProviderBase');
AriKernel::import('Application.ARISmartContent.Utils.ApplicationHelper');
AriKernel::import('Picasa.Picasa');
AriKernel::import('Utils.Utils');
AriKernel::import('SimpleTemplate.SimpleTemplate');
AriKernel::import('Web.JSON.JSON');
AriKernel::import('Parameters.ParametersHelper');

class AriSmartContentProviderPicasaImages extends AriSmartContentProviderBase
{
	function getPicasaObject($source)
	{
		$params = $source->Params;
		$cachePeriod = intval(AriUtils::getParam($params, 'cachePeriod', 10), 10);
		
		$helper = AriSmartContentApplicationHelper::getInstance();
		$cacheFolder = $helper->getCacheFolder();
		
		$picasa = new AriPicasa();
		if ($cachePeriod > 0 && $cacheFolder)
			$picasa->enableCache($cacheFolder, $cachePeriod);
		
		return $picasa;
	}
	
	function getData($source, $filter)
	{
		$data = array();
		$params = $source->Params;
		$picasa = $this->getPicasaObject($source);		

		if ($filter)
		{
			$filterOffset = $filter->getConfigValue('startOffset');
			$filterLimit = $filter->getConfigValue('limit');

			if (!is_null($filterOffset))
				$params->offset = $filterOffset + 1;

			if (!is_null($filterLimit))
				$params->count = $filterLimit;
		}

		$src = 'album';
		$paramsTree = AriParametersHelper::flatParametersToArray($params);
		$srcParams = isset($paramsTree['source'][$src]) ? $paramsTree['source'][$src] : array();

		$data = $this->getUserAlbumData($srcParams['user'], $srcParams['album'], $params, $picasa);

		return $data;
	}

	function getTotalRecords($source, $filter)
	{
		$params = $source->Params;
		$picasa = $this->getPicasaObject($source);

		$src = 'album';
		$paramsTree = AriParametersHelper::flatParametersToArray($params);
		$srcParams = isset($paramsTree['source'][$src]) ? $paramsTree['source'][$src] : array();

		$totalRecords = $picasa->getUserAlbumTotalRecords($srcParams['user'], $srcParams['album'], array());

		return $totalRecords;
	}

	function getUserAlbumData($user, $album, $params, $picasa)
	{
		$data = array();
		$options = array(
			'imgmax' => AriUtils::getParam($params, 'itemSize', 1600)
		);

		$previewSize = AriUtils::getParam($params, 'previewSize');
		$navSize = AriUtils::getParam($params, 'navSize');
		
		$thumbSizeOpt = array();
		
		if ($previewSize)
			$thumbSizeOpt[] = $previewSize;
			
		if ($navSize)
			$thumbSizeOpt[] = $navSize;
		
		if (count($thumbSizeOpt) > 0)
			$options['thumbsize'] = join(',', $thumbSizeOpt);

		$maxResults = intval(AriUtils::getParam($params, 'count'), 10);
		if ($maxResults > 0) 
			$options['max-results'] = $maxResults;

		$offset = intval(AriUtils::getParam($params, 'offset'), 10);
		if ($offset > 0) 
			$options['start-index'] = $offset;

		$data = $picasa->getUserAlbumData($user, $album, $options);
		$data = $this->normalizeFeedData($data, $params);

		return $data;
	}
	
	function normalizeFeedData($data, $params)
	{
		$normalizedData = array();

		if (empty($data))
			return $normalizedData;

		$previewSize = AriUtils::getParam($params, 'previewSize');
		$navSize = AriUtils::getParam($params, 'navSize');

		foreach ($data as $dataItem)
		{
			$img = $dataItem['image'];
			
			$newDataItem = array(
				'summary' => $dataItem['summary'],
				'Title' => $dataItem['title'],
				'Item_Url' => $img['url'],
				'Item_Width' => $img['w'],
				'Item_Height' => $img['h'],
			);
			
			$thumb = AriUtils::getParam($dataItem, 'thumb');
			if (!empty($thumb))
			{
				if (isset($thumb[$previewSize]))
				{
					$previewThumb = $thumb[$previewSize];
					$newDataItem['Preview_Url'] = $previewThumb['url'];
					$newDataItem['Preview_Width'] = $previewThumb['w'];
					$newDataItem['Preview_Height'] = $previewThumb['h'];
				}

				if (isset($thumb[$navSize]))
				{
					$navThumb = $thumb[$navSize];
					$newDataItem['Nav_Url'] = $navThumb['url'];
					$newDataItem['Nav_Width'] = $navThumb['w'];
					$newDataItem['Nav_Height'] = $navThumb['h'];
				}
			}
			
			$normalizedData[] = $newDataItem;
		}
			
		return $normalizedData;
	}
	
	function getError($i = null, $toString = true)
	{
		return $this->_picasa->getError($i, $toString);
	}
}