<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

require_once JPATH_ADMINISTRATOR . '/components/com_arismartcontent/models/providersource.php';
require_once JPATH_ADMINISTRATOR . '/components/com_arismartcontent/models/providersettings.php';

AriKernel::import('Application.ARISmartContent.Formatters.FormattersHelper');

class AriSmartContentProvidersHelper
{
	static function resolveSource($sourceId, $overrideSourceParams = array())
	{
		$data = null;

		$sourceModel = AriModel::getInstance('Providersource', 'AriSmartContentModel');
		$source = $sourceModel->getSource($sourceId);

		if (empty($source))
			return $data;


        if (is_array($overrideSourceParams) && count($overrideSourceParams) > 0)
        {
            foreach ($overrideSourceParams as $overrideSourceParam => $overrideSourceParamValue)
            {
                $source->Params->$overrideSourceParam = $overrideSourceParamValue;
            }
        }

		// load default settings
		if ($source->SettingsId == 0 && $source->SourceType->SettingsGroupId > 0)
		{
			$settingsModel = AriModel::getInstance('Providersettings', 'AriSmartContentModel');
			$defaultSettings = $settingsModel->getDefaultSettings($source->SourceType->SettingsGroupId);

			if (empty($defaultSettings) || empty($defaultSettings->Id))
				return $data;
				
			$source->SettingsId = $defaultSettings->Id;
			$source->ProviderSettings = $defaultSettings;
		}
			
		$providerName = $source->SourceType->getProviderName();
		$providerPostfix = $source->SourceType->getPostfix();
		$provider = self::getProvider($providerName, $providerPostfix);
		if (is_null($provider))
			return $data;
			
		$data = array(
			'source' => $source,
			'provider' => $provider,
			'columns' => array(),
			'settings' => array()
		);

		$formatter = null;
		if ($source->Formatter && $source->Formatter->Type)
			$formatter = AriSmartContentFormattersHelper::getFormatter($source->Formatter->Type);

		$columns = $provider->getColumns($source);
		$settings = array();

		if ($formatter)
		{
			$formatterSettings = $source->Formatter->Data;

			$settings = $formatter->getColumnsSettings($formatterSettings, $columns);
		}

		$data['columns'] = $columns;
		$data['settings'] = $settings;

		return $data;
	}
	
	static function getData($sourceData, $filter)
	{
		$data = null;
		if (empty($sourceData))
			return $data;

		$source = $sourceData['source'];
		$provider = $sourceData['provider'];
		$columns = $sourceData['columns'];
		$settings = $sourceData['settings'];

		$formatter = null;
		if ($source->Formatter && $source->Formatter->Type)
			$formatter = AriSmartContentFormattersHelper::getFormatter($source->Formatter->Type);

		$data = $provider->getData($source, $filter);		
		if ($formatter)
		{
			$formatterSettings = $source->Formatter->Data;			
			$data = $formatter->formatData($data, $formatterSettings, $columns, $settings);
		}

		return $data;
	}

	static function getTotalRecords($sourceData, $filter)
	{
		if (empty($sourceData))
			return 0;

		$source = $sourceData['source'];
		$provider = $sourceData['provider'];
		
		return $provider->getTotalRecords($sourceData['source'], $filter);
	}

	static function getProvider($providerName, $providerPostfix)
	{
		$providerPath = 'Application.ARISmartContent.Providers.' . ucfirst($providerName) . '.' . $providerPostfix . 'Provider';

		AriKernel::import($providerPath);
		
		$providerClass = 'AriSmartContentProvider' . $providerPostfix;
		if (!class_exists($providerClass))
		{
			JError::raiseError(
				500,
				sprintf(
					'ARI Smart Content: "%1$s" provider does not exist',
					$providerClass
				)
			);
			
			return null;
		}
		
		$provider = new $providerClass();
		
		return $provider;
	}
	
	static function getColumnsNames($columns)
	{
		$names = array();
		
		if (!is_array($columns))
			return $names;
			
		foreach ($columns as $col)
		{
			$names[] = $col['name'];
		}
		
		return $names;
	}
	
	static function isSupportServerSide($provider, $source)
	{
		return $provider->isSupportServerSide($source);
	}
}