<?php
/*
 *
 * @package		ARI Framework
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die('Direct Access to this location is not allowed.');

AriKernel::import('Utils.Utils');
AriKernel::import('SimpleTemplate.SimpleTemplate');
AriKernel::import('Plugin.ContentPlugin');

class AriRepeaterWebControl
{
	var $_template;
	var $_data;
	var $_tag;
	
	function __construct($template, $data, $tag = 'repeater')
	{
		$this->_template = $template;
		$this->_data = $data;
		$this->_tag = $tag;
	}
	
	function getContent($attrs = null)
	{
		$rpt = new AriRepeaterMainTemplate($this->_tag);
		$rpt->parse($this->_template);
		
		return $rpt->render($this->_data);
	}

	function render($attrs = null)
	{
		echo $this->getContent($attrs);
	}
}

class AriRepeaterMainTemplate extends AriContentPlugin
{
	var $_headerTemplate;
	var $_footerTemplate;
	var $_rowTemplate;
	var $_cellTemplate;
	var $_emptyCellTemplate;
	var $_emptyTemplate;
	
	function __construct($tag = null, $nested = false)
	{
		$this->_headerTemplate = new AriRepeaterGenericTemplate('headertemplate');
		$this->_footerTemplate = new AriRepeaterGenericTemplate('footertemplate');
		$this->_emptyTemplate = new AriRepeaterGenericTemplate('emptytemplate');
		$this->_cellTemplate = new AriRepeaterCellTemplate('celltemplate');
		$this->_emptyCellTemplate = new AriRepeaterGenericTemplate('emptycelltemplate');
		$this->_rowTemplate = new AriRepeaterRowTemplate('rowtemplate');

		if (empty($tag))
			$tag = 'repeater';

		parent::__construct($tag, false);
	}
	
	function contentHandler($params, $content, $sourceContent) 
	{
		$content = $this->_headerTemplate->parse($content);
		$content = $this->_footerTemplate->parse($content);
		$content = $this->_emptyTemplate->parse($content);
		$content = $this->_cellTemplate->parse($content);
		$content = $this->_emptyCellTemplate->parse($content);
		$content = $this->_rowTemplate->parse($content);

		return '';
	}

	function render($data)
	{
		if (!is_array($data) || count($data) == 0)
			return $this->_emptyTemplate->getContent();

		$rowTemplate = $this->_rowTemplate->getContent();
		$cellTemplate = $this->_cellTemplate->getContent();
		$emptyCellTemplate = $this->_emptyCellTemplate->getContent();
		$content = $this->_headerTemplate->getContent();
		$columnPerRow = $this->_rowTemplate->getColumnCount();
		$itemIndex = 0;
		$rowIndex = 0;
		$columns = '';
		foreach ($data as $dataItem)
		{
			$columns .= AriSimpleTemplate::parse($cellTemplate, array('data' => $dataItem), true);
			++$itemIndex;
			
			if ($itemIndex && ($itemIndex % $columnPerRow == 0))
			{
				$content .= AriSimpleTemplate::parse(
					str_replace('#{cellTemplate}', $columns, $rowTemplate), 
					array('rowClass' => $this->_rowTemplate->getRowClass($rowIndex)));
				$columns = '';
				++$rowIndex;
			}
		}

		if ($itemIndex % $columnPerRow != 0)
		{
			$columns .= str_repeat($emptyCellTemplate, $columnPerRow - ($itemIndex % $columnPerRow));
			$content .= AriSimpleTemplate::parse(
				str_replace('#{cellTemplate}', $columns, $rowTemplate), 
				array('rowClass' => $this->_rowTemplate->getRowClass($rowIndex)));
		}
		
		$content .= $this->_footerTemplate->getContent();

		return $content;
	}
}

class AriRepeaterGenericTemplate extends AriContentPlugin
{
	var $_content = '';
	
	function __construct($tag)
	{
		parent::__construct($tag);
	}

	function contentHandler($params, $content, $sourceContent) 
	{
		$this->_content = trim($content);

		return '';
	}

	function getContent()
	{
		return $this->_content;
	}
}

class AriRepeaterCellTemplate extends AriRepeaterGenericTemplate 
{
	function contentHandler($params, $content, $sourceContent) 
	{
		parent::contentHandler($params, $content, $sourceContent);
		
		return '#{cellTemplate}';
	}
}

class AriRepeaterRowTemplate extends AriRepeaterGenericTemplate 
{
	var $_columnCount = 1;
	var $_rowClasses = array();
	
	function contentHandler($params, $content, $sourceContent) 
	{
		$this->_columnCount = intval(AriUtils::getParam($params, 'itemCount', 1), 10);
		$rowClasses = AriUtils::getParam($params, 'rowClass', '');
		$rowClasses = explode(';', $rowClasses);
		array_walk($rowClasses, 'trim');
		$this->_rowClasses = $rowClasses;
		
		return parent::contentHandler($params, $content, $sourceContent);
	}
	
	function getColumnCount()
	{
		return $this->_columnCount;
	}
	
	function getRowClass($rowIndex)
	{
		$rowClasses = $this->_rowClasses;
		$cnt = count($rowClasses);
		if ($cnt < 1)
			return '';

		return $rowClasses[$rowIndex % $cnt];
	}
}