<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

AriKernel::import('Joomla.Models.Model');
AriKernel::import('Joomla.Database.DBUtils');
AriKernel::import('Utils.ArrayHelper');

class AriSmartContentModelProvidersettings extends AriModel 
{
	function getSettingsCount($group, $filter = null)
	{
		$db =& $this->getDBO();

		$query = AriDBUtils::getQuery();
		$query->select('COUNT(*)');
		$query->from('#__arismartcontent_provider_settings S');
		$query->innerJoin('#__arismartcontent_provider_settings_groups G ON S.GroupId = G.GroupId');
		$query->where(
			sprintf(
				'G.SystemName = %1$s',
				$db->Quote($group)
			)
		);

		if ($filter)
			$query = $this->_applyFilter($query, $filter);

		$db->setQuery((string)$query);
		$count = $db->loadResult();

		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$query, 
					$db->getErrorMsg()
				)
			);
			
			return 0;
		}

		return $count;
	}
	
	function getSettingsList($group, $filter = null)
	{
		$db =& $this->getDBO();
		if (empty($filter))
			$filter = new AriDataFilter();

		$query = AriDBUtils::getQuery();
		$query->select(
			array(
				'S.Id',
				'S.Name',
				'S.Description',
				'S.Default'
			)
		);
		$query->from('#__arismartcontent_provider_settings S');
		$query->innerJoin('#__arismartcontent_provider_settings_groups G ON S.GroupId = G.GroupId');
		$query->where(
			sprintf(
				'G.SystemName = %1$s',
				$db->Quote($group)
			)
		);
		
		if ($filter)
		{
			$query = $this->_applyFilter($query, $filter);
			$query = $filter->applyToQuery($query);
		}

		$db->setQuery((string)$query, $filter->getConfigValue('startOffset'), $filter->getConfigValue('limit'));
		$groups = $db->loadObjectList();
		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$query, 
					$db->getErrorMsg()
				)
			);
			
			return null;
		}
		
		return $groups;
	}
	
	function delete($group, $idList)
	{
		$idList = AriArrayHelper::toInteger($idList, 1);
		if (count($idList) == 0) 
			return false;
		
		$db =& $this->getDBO();
		
		$idListStr = join(',', $idList);
		$query = sprintf('DELETE S FROM #__arismartcontent_provider_settings S INNER JOIN #__arismartcontent_provider_settings_groups G ON S.GroupId = G.GroupId WHERE G.SystemName = %1$s AND S.Id IN (%2$s)',
			$db->Quote($group),
			$idListStr
		);

		$db->setQuery($query);
		$db->query();
		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$db->getQuery(), 
					$db->getErrorMsg()
				)
			);
				
			return false;
		}

		return true;
	}
	
	function markAsDefault($group, $id)
	{
		if ($id < 1)
			return false;
			
		$db = $this->getDBO();
		
		$idListStr = join(',', $idList);
		$query = sprintf('SELECT COUNT(*) FROM #__arismartcontent_provider_settings S INNER JOIN #__arismartcontent_provider_settings_groups G ON S.GroupId = G.GroupId WHERE S.Id = %2$d AND G.SystemName = %1$s LIMIT 0,1',
			$db->Quote($group),
			$id
		);
		$db->setQuery($query);
		$cnt = $db->loadResult();
		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$db->getQuery(), 
					$db->getErrorMsg()
				)
			);
				
			return false;
		}
		
		if ($cnt != 1)
			return false;
			
		$queryList = array(
			sprintf(
				'UPDATE #__arismartcontent_provider_settings S INNER JOIN #__arismartcontent_provider_settings_groups G ON S.GroupId = G.GroupId SET S.`Default` = 0 WHERE G.SystemName = %1$s',
				$db->Quote($group)
			),
			sprintf(
				'UPDATE #__arismartcontent_provider_settings SET `Default` = 1 WHERE Id = %1$d',
				$id
			)
		);
		
		foreach ($queryList as $query)
		{
			$db->setQuery($query);
			$db->query();
			if ($db->getErrorNum())
			{
				JError::raiseError(
					500, 
					JText::sprintf(
						'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
						__CLASS__ . '::' . __FUNCTION__ . '()', 
						$db->getQuery(), 
						$db->getErrorMsg()
					)
				);
					
				return false;
			}			
		}

		return true;
	}
	
	function getSettings($id, $group = null, $strictLoad = true) 
	{
		if ($strictLoad && $id < 1)
			return null;

		$settings = $this->getTable();
		$settings->load($id);
		
		if (
			($strictLoad && empty($settings->Name)) ||
			(!is_null($group) && $settings->Group->SystemName != $group)
		)
			$settings = null;

		return $settings;
	}
	
	function saveSettings($data, $settings, $group)
	{
		if (!is_array($data))
			$data = array();

		$groupModel = AriModel::getInstance('Providergroup', $this->getFullPrefix());
		$providerGroup = $groupModel->getGroupBySystemName($group);
		
		if (empty($providerGroup) || empty($providerGroup->GroupId))
			return null;

		$data['Params'] = $settings;
		$data['GroupId'] = $providerGroup->GroupId;

		$settingsTable = $this->getTable();
		$settingsTable->bind($data);

		if (!$settingsTable->store())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$settingsTable->getQuery(), 
					$settingsTable->getError()
				)
			);
			return null;
		}
		
		return $settingsTable;
	}
	
	function isUniqueName($name, $id, $group)
	{
		$db = $this->getDBO();

		$query = AriDBUtils::getQuery();
		$query->select('COUNT(*)');
		$query->from('#__arismartcontent_provider_settings S');
		$query->innerJoin('#__arismartcontent_provider_settings_groups G ON S.GroupId = G.GroupId');

		$query->where('S.Name = ' . $db->Quote($name));
		if ($id)
			$query->where('S.Id <> ' . intval($id, 10));

		$query->where(
			sprintf(
				'G.SystemName = %1$s',
				$db->Quote($group)
			)
		);	
			
		$db->setQuery((string)$query);
			
		$isUnique = $db->loadResult();
		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$query, 
					$db->getErrorMsg()
				)
			);
			return null;
		}
		
		return ($isUnique == 0);
	}
	
	function getDefaultSettings($groupId, $strictLoad = true)
	{
		if ($strictLoad && empty($groupId))
			return null;

		$settings = $this->getTable();
		$settings->loadDefault($groupId);
		
		if (
			($strictLoad && empty($settings->Name))
		)
			$settings = null;

		return $settings;
		
	}
	
	function _applyFilter($query, $filter)
	{
		if (empty($filter))
			return $query;		
					
		$filterPredicates = $filter->getConfigValue('filter');
		
		return $query;
	}
}