<?php
/*
 *
 * @package		ARI Smart Content
 * @author		ARI Soft
 * @copyright	Copyright (c) 2011 www.ari-soft.com. All rights reserved
 * @license		GNU/GPL (http://www.gnu.org/copyleft/gpl.html)
 * 
 */

(defined('_JEXEC') && defined('ARI_FRAMEWORK_LOADED')) or die;

AriKernel::import('Joomla.Models.Model');
AriKernel::import('Joomla.Database.DBUtils');
AriKernel::import('Utils.ArrayHelper');

require_once dirname(__FILE__) . '/../tables/providersource.php';

class AriSmartContentModelProvidersource extends AriModel 
{
	function getSourceCount($type, $filter = null)
	{
		$db =& $this->getDBO();

		$query = AriDBUtils::getQuery();
		$query->select('COUNT(*)');
		$query->from('#__arismartcontent_provider_sources S');
		$query->innerJoin('#__arismartcontent_provider_source_types T ON S.SourceTypeId = T.SourceTypeId');
		$query->where(
			sprintf(
				'T.SystemName = %1$s',
				$db->Quote($type)
			)
		);

		if ($filter)
			$query = $this->_applyFilter($query, $filter);

		$db->setQuery((string)$query);
		$count = $db->loadResult();

		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$query, 
					$db->getErrorMsg()
				)
			);
			
			return 0;
		}

		return $count;
	}
	
	function getSourceList($type, $filter = null)
	{
		$db =& $this->getDBO();
		if (empty($filter))
			$filter = new AriDataFilter();

		$query = AriDBUtils::getQuery();
		$query->select(
			array(
				'S.Id',
				'S.Name',
				'S.Description'
			)
		);
		$query->from('#__arismartcontent_provider_sources S');
		$query->innerJoin('#__arismartcontent_provider_source_types T ON S.SourceTypeId = T.SourceTypeId');
		$query->where(
			sprintf(
				'T.SystemName = %1$s',
				$db->Quote($type)
			)
		);
		
		if ($filter)
		{
			$query = $this->_applyFilter($query, $filter);
			$query = $filter->applyToQuery($query);
		}

		$db->setQuery((string)$query, $filter->getConfigValue('startOffset'), $filter->getConfigValue('limit'));
		$sources = $db->loadObjectList();
		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$query, 
					$db->getErrorMsg()
				)
			);
			
			return null;
		}
		
		return $sources;
	}
	
	function delete($type, $idList)
	{
		$idList = AriArrayHelper::toInteger($idList, 1);
		if (count($idList) == 0) 
			return false;
		
		$db =& $this->getDBO();
		
		$idListStr = join(',', $idList);
		$query = sprintf('DELETE S FROM #__arismartcontent_provider_sources S INNER JOIN #__arismartcontent_provider_source_types T ON S.SourceTypeId = T.SourceTypeId WHERE T.SystemName = %1$s AND S.Id IN (%2$s)',
			$db->Quote($type),
			$idListStr
		);

		$db->setQuery($query);
		$db->query();
		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$db->getQuery(), 
					$db->getErrorMsg()
				)
			);
				
			return false;
		}

		return true;
	}
	
	function getSource($id, $type = null, $strictLoad = true) 
	{
		if ($strictLoad && $id < 1)
			return null;

		$source = $this->getTable();
		$source->load($id);

		if (
			($strictLoad && empty($source->Name)) ||
			(!is_null($type) && $source->SourceType->SystemName != $type)
		)
			$source = null;

		return $source;
	}
	
	function saveSource($data, $source, $type, $formatterData)
	{
		if (!is_array($data))
			$data = array();

		$sourceTypeModel = AriModel::getInstance('Providersourcetype', $this->getFullPrefix());
		$sourceType = $sourceTypeModel->getTypeBySystemName($type);

		if (empty($sourceType) || empty($sourceType->SourceTypeId))
			return null;

		$data['Params'] = $source;
		$data['SourceTypeId'] = $sourceType->SourceTypeId;

		$sourceTable = null;
		if (!empty($data['Id']))
		{
			$sourceTable = $this->getSource($data['Id'], $type);

			if (is_null($sourceTable))
				return null;
				
			$data['FormatterId'] = $sourceTable->FormatterId;
		}
		else
		{
			$sourceTable = $this->getTable();
		}

		$data['Formatter'] = $formatterData;

		$sourceTable->bind($data);
		if (!$sourceTable->store())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$sourceTable->getQuery(), 
					$sourceTable->getError()
				)
			);
			return null;
		}
		
		return $sourceTable;
	}
	
	function isUniqueName($name, $id, $type)
	{
		$db = $this->getDBO();

		$query = AriDBUtils::getQuery();
		$query->select('COUNT(*)');
		$query->from('#__arismartcontent_provider_sources S');
		$query->innerJoin('#__arismartcontent_provider_source_types T ON S.SourceTypeId = T.SourceTypeId');
		$query->where('S.Name = ' . $db->Quote($name));
		if ($id)
			$query->where('S.Id <> ' . intval($id, 10));

		$query->where(
			sprintf(
				'T.SystemName = %1$s',
				$db->Quote($type)
			)
		);		

		$db->setQuery((string)$query);
			
		$isUnique = $db->loadResult();
		if ($db->getErrorNum())
		{
			JError::raiseError(
				500, 
				JText::sprintf(
					'COM_ARISMARTCONTENT_ERROR_SQL_QUERY', 
					__CLASS__ . '::' . __FUNCTION__ . '()', 
					$query, 
					$db->getErrorMsg()
				)
			);
			return null;
		}
		
		return ($isUnique == 0);
	}

    function getUniqueName($name)
    {
        $name = trim($name);
        if (empty($name))
            $name = 'New source';
        else
            $name = preg_replace('/\s\(\s*[\s0-9]+\s*\)$/', '', $name);

        $db = $this->getDBO();

        $db->setQuery(
            sprintf(
                'SELECT `Name` FROM #__arismartcontent_provider_sources WHERE `Name` LIKE CONCAT(%1$s," (%%)")',
                $db->Quote($name)
            )
        );
        echo $db->getQuery();
        $names = $db->loadAssocList();

        $postfix = 1;
        if (is_array($names) && count($names) > 0)
        {
            $postfixList = array();
            foreach ($names as $srcName)
            {
                $matches = array();
                if (preg_match('/\(\s*(\d+)\s*\)$/', $srcName['Name'], $matches))
                    $postfixList[] = $matches[1];
            }

            sort($postfixList, SORT_NUMERIC);
            $postfix = $postfixList[count($postfixList) - 1] + 1;
        }

        return $name . ' (' . $postfix . ')';
    }
	
	function _applyFilter($query, $filter)
	{
		if (empty($filter))
			return $query;		
					
		$filterPredicates = $filter->getConfigValue('filter');
		
		return $query;
	}
}